package com.sf.service.gateway.domain.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.sf.common.annotation.Excel;
import com.sf.file.domain.SysOss;
import com.sf.service.gateway.domain.GatewayInterfaceInfo;
import com.sf.service.gateway.domain.GatewayInterfaceMock;
import com.sf.service.gateway.domain.GatewayServer;
import com.sf.service.gateway.domain.GatewayStrategy;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.Pattern;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import static org.apache.commons.lang3.StringUtils.EMPTY;

/**
 * 接口信息详细展示
 *
 * @author zoukun
 * @date 2024-04-22
 */
@Data
@ApiModel(value = "GatewayInterfaceInfoDetailedVO", description = "接口信息详细展示对象")
public class GatewayInterfaceInfoDetailedVO {
    private static final long serialVersionUID = 1L;

    /**
     * ID
     */
    @ApiModelProperty("ID")
    private Long id;

    /**
     * 接口名称
     */
    @Excel(name = "接口名称")
    @ApiModelProperty("接口名称")
    private String interfaceName;

    /**
     * api编码
     */
    @Excel(name = "接口编码")
    @ApiModelProperty("接口编码")
    private String apiCode;

    /**
     * 接口路径
     */
    @Excel(name = "接口路径")
    @ApiModelProperty("接口名称")
    private String interfacePath;

    /**
     * 请求方式
     */
    @Excel(name = "请求方式")
    @ApiModelProperty("请求方式")
    private String requestMethod;

    /**
     * 接口描述
     */
    @Excel(name = "接口描述")
    @ApiModelProperty("接口描述")
    private String description;

    /**
     * 接口版本
     */
    @Excel(name = "接口版本")
    @ApiModelProperty("接口版本")
    private String version;

    /**
     * 接口状态（0停用 1启用）
     */
    @Excel(name = "接口状态", readConverterExp = "0=停用,1=启用")
    @ApiModelProperty("接口状态(0=停用,1=启用)")
    private String status;

    /**
     * 接口文档（文档地址）
     */
    @Excel(name = "接口文档", readConverterExp = "文档地址")
    @ApiModelProperty("接口文档（文档地址）")
    private SysOss document;

    @ApiModelProperty("调用次数")
    private Long numberOfCalls;

    @ApiModelProperty("响应时间（平均 ms）")
    private Long averageResponseTime;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date createTime;

    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date updateTime;

    @ApiModelProperty("网关服务id")
    private Long serverId;

    @ApiModelProperty("网关服务名称")
    private String serverName;

    @ApiModelProperty("超时时间，单位毫秒,1-150000之间")
    private Integer timeout;

    @ApiModelProperty
    private List<GatewayStrategyListVO> strategyList;

    @ApiModelProperty("Mock状态(0=停用,1=启用)")
    private String mockStatus;

    @ApiModelProperty("Mock默认响应HTTP状态码")
    private Integer mockDefaultHttpStatus;

    @ApiModelProperty("Mock默认响应，json格式")
    private String mockDefaultResponse;

    @ApiModelProperty("Mock默认响应，json格式")
    private List<GatewayInterfaceMockVO> mockExpectations;


    public static GatewayInterfaceInfoDetailedVO convert(GatewayInterfaceInfo gatewayInterfaceInfo
            , GatewayServer gatewayServer
            , List<GatewayStrategy> strategyList
            , SysOss document
            , List<GatewayInterfaceMock> mockExpectations

    ) {
        GatewayInterfaceInfoDetailedVO vo = new GatewayInterfaceInfoDetailedVO();
        vo.setId(gatewayInterfaceInfo.getId());
        vo.setInterfaceName(gatewayInterfaceInfo.getInterfaceName());
        vo.setApiCode(gatewayInterfaceInfo.getApiCode());
        vo.setInterfacePath(gatewayInterfaceInfo.getInterfacePath());
        vo.setRequestMethod(gatewayInterfaceInfo.getRequestMethod());
        vo.setDescription(gatewayInterfaceInfo.getDescription());
        vo.setVersion(gatewayInterfaceInfo.getVersion());
        vo.setStatus(gatewayInterfaceInfo.getStatus());
        vo.setDocument(document);
        vo.setCreateTime(gatewayInterfaceInfo.getCreateTime());
        vo.setUpdateTime(gatewayInterfaceInfo.getUpdateTime());
        vo.setServerId(gatewayInterfaceInfo.getServerId());
        vo.setServerName(gatewayServer.getServerName());
        vo.setTimeout(gatewayInterfaceInfo.getTimeout());
        vo.setMockStatus(gatewayInterfaceInfo.getMockStatus());
        vo.setMockDefaultHttpStatus(gatewayInterfaceInfo.getMockDefaultHttpStatus());
        vo.setMockDefaultResponse(gatewayInterfaceInfo.getMockDefaultResponse());
        // mock配置
        List<GatewayInterfaceMockVO> mockExpectationVOs = mockExpectations.stream().map(GatewayInterfaceMockVO::convert).collect(Collectors.toList());
        vo.setMockExpectations(mockExpectationVOs);
        // 策略信息
        List<GatewayStrategyListVO> strategyListVOList = strategyList.stream()
                .map(GatewayStrategyListVO::convert)
                .collect(Collectors.toList());
        vo.setStrategyList(strategyListVOList);
        return vo;
    }
}
