package com.sf.vertx.handle;

import static com.sf.vertx.constans.SACConstants.API_SERVICE_TYPE;
import static com.sf.vertx.constans.SACConstants.CACHE_KEY_CONNECTOR;
import static org.apache.commons.lang3.StringUtils.EMPTY;

import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang3.StringUtils;

import com.sf.vertx.api.pojo.ApiConfig;
import com.sf.vertx.api.pojo.AppConfig;
import com.sf.vertx.enums.GatewayError;
import com.sf.vertx.enums.RequestMethod;
import com.sf.vertx.exception.ServiceException;
import com.sf.vertx.pojo.SacCircuitBreaker;
import com.sf.vertx.utils.AppUtils;
import com.sf.vertx.utils.ProxyTool;

import cn.hutool.core.util.StrUtil;
import io.vertx.circuitbreaker.CircuitBreaker;
import io.vertx.circuitbreaker.OpenCircuitException;
import io.vertx.core.buffer.Buffer;
import io.vertx.core.http.HttpHeaders;
import io.vertx.core.impl.NoStackTraceThrowable;
import io.vertx.core.json.JsonObject;
import io.vertx.core.net.SocketAddress;
import io.vertx.ext.web.RoutingContext;
import io.vertx.ext.web.client.HttpRequest;
import io.vertx.ext.web.client.WebClient;
import io.vertx.ext.web.handler.HttpException;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class SacRouteRequestHandlerImpl implements SacRouteRequestHandler {
	private WebClient mainWebClient;

	public SacRouteRequestHandlerImpl(WebClient mainWebClient) {
		this.mainWebClient = mainWebClient;
	}

	private void breakerCall(RoutingContext ctx, SacCircuitBreaker sacCircuitBreaker, HttpRequest<Buffer> requestBuffer,
			String body) {
		String appCode = AppUtils.getAppConfigFromRoutingContext(ctx).getAppCode();
		sacCircuitBreaker.getCircuitBreaker().executeWithFallback(promise -> {
			requestBuffer.sendJson(body, h -> {
				if (h.succeeded()) {
					log.info("==========uri:{},response http code:{}", ctx.request().uri(), h.result().statusCode());
					if (h.result().statusCode() == 200) {
						// promise.complete();
						promise.complete("1");
					} else {
						// Throwable throwable = new Throwable("error port");
						// promise.fail(throwable);
						promise.fail("2");
					}
					JsonObject responseData = h.result().bodyAsJsonObject();
					//log.info("responseData:{}", responseData);
					Buffer buffer;
					// 加密
					if (AppConfigHandler.isDataSecurity(appCode)) {
						String dataStr = AppConfigHandler.bodyEncrypt(responseData.toString(), appCode);
						//log.info("encrypt dataStr:{}", dataStr);
						buffer = Buffer.buffer(dataStr);
					} else {
						buffer = responseData.toBuffer();
					}

					Integer statusCode = h.result().statusCode() > 0 ? h.result().statusCode() : 200;
					// TODO 和客户端组件沟通
//						String contentType = h.result().headers().get(HttpHeaders.CONTENT_TYPE) != null
//						? h.result().headers().get(HttpHeaders.CONTENT_TYPE)
//						: "application/json";
					String contentType = "application/json";
					ctx.response().setChunked(true).setStatusCode(statusCode)
							.putHeader(HttpHeaders.CONTENT_TYPE, contentType).end(buffer);
				} else {
					// end(proxyRequest, 502);
					// Throwable throwable = new Throwable("error port");
					// promise.fail(throwable);
					promise.fail("3");
				}
			});
		}, v -> {
			log.info(sacCircuitBreaker.getCircuitBreaker().name() + " executed when the circuit is opened:{}", v);
			if (v instanceof OpenCircuitException) {
				// v value is null, default return 3
				log.info(sacCircuitBreaker.getCircuitBreaker().name() + " open circuit");
			} else if (v instanceof NoStackTraceThrowable) {
				log.info(sacCircuitBreaker.getCircuitBreaker().name() + " close circuit");
				return v.getMessage();
			} else {
				log.info(sacCircuitBreaker.getCircuitBreaker().name() + " half open circuit");
				return v.getMessage();
			}
			return "3";
		}, ar -> {
			log.info(sacCircuitBreaker.getCircuitBreaker().name() + " interface failed result.{} ", ar);
			if (StringUtils.equals(ar.result(), "3")) { // 全开,熔断
				ctx.fail(new ServiceException(GatewayError.REQUEST_URL_IS_BROKEN, sacCircuitBreaker.getStrategy().getDefaultResponse()));
			}
		});
	}

	private void normalCall(RoutingContext ctx, HttpRequest<Buffer> requestBuffer, String body) {
		String appCode = AppUtils.getAppConfigFromRoutingContext(ctx).getAppCode();
		requestBuffer.sendJson(body, h -> {
			if (h.succeeded()) {
				log.info("==========uri:{},response http code:{}", ctx.request().uri(), h.result().statusCode());
				if (h.result().statusCode() == 200) {
					JsonObject responseData = h.result().bodyAsJsonObject();
					log.info("responseData:{}", responseData);
					Buffer bodyBuffer = responseData.toBuffer();
					// 加密
					if (AppConfigHandler.isDataSecurity(appCode)) {
						String dataStr = AppConfigHandler.bodyEncrypt(responseData.toString(), appCode);
						log.info("encrypt dataStr:{}", dataStr);
						bodyBuffer = Buffer.buffer(dataStr);
					}
					ctx.response().setChunked(true).setStatusCode(200).putHeader("Content-Type", "application/json")
							.end(bodyBuffer);
				}
			} else {
				log.info("interface retrun error.{}", ctx.request().uri());
				ctx.fail(new ServiceException(GatewayError.BAD_GATEWAY));
			}
		});
	}

	private HttpRequest<Buffer> buildCallRequestBuffer(RoutingContext ctx, String body, ApiConfig apiConfig) {
		SocketAddress socketAddress = ProxyTool.resolveOriginAddress(ctx);
		HttpRequest<Buffer> requestBuffer;
		String requestURI = apiConfig.getUri();
		switch (apiConfig.getMethod()) {
		case "PUT":
			requestBuffer = mainWebClient.put(socketAddress.port(), socketAddress.host(), requestURI);
			break;
		case "DELETE":
			requestBuffer = mainWebClient.delete(socketAddress.port(), socketAddress.host(), requestURI);
			break;
		case "HEAD":
			requestBuffer = mainWebClient.head(socketAddress.port(), socketAddress.host(), requestURI);
			break;
		case "GET":
			requestBuffer = mainWebClient.get(socketAddress.port(), socketAddress.host(), requestURI);
			break;
		default:
			requestBuffer = mainWebClient.post(socketAddress.port(), socketAddress.host(), requestURI);
			break;
		}
		requestBuffer.timeout(apiConfig.getTimeout()); // 超时
		requestBuffer.putHeaders(ctx.request().headers());
		// 处理sac请求方式/{a}/{b}
		RequestMethod requestMethod = RequestMethod.getByCode(ctx.request().method().name());
		if (RequestMethod.GET.equals(requestMethod) || RequestMethod.DELETE.equals(requestMethod)
				|| RequestMethod.HEAD.equals(requestMethod)) {
			JsonObject jsonBody = new JsonObject(body);
			Map<String, String> queryParams = new HashMap<>();
			for (Map.Entry<String, Object> entry : jsonBody) {
				String val = entry.getValue() != null ? entry.getValue().toString() : EMPTY;
				queryParams.put(entry.getKey(), val);
				requestURI = StrUtil.replace(requestURI, "{" + entry.getKey() + "}", val);
			}
			requestBuffer.uri(requestURI);
			requestBuffer.queryParams().addAll(queryParams);
		}
		return requestBuffer;
	}

	@Override
	public void handle(RoutingContext ctx) {
		try {
			// 判断body是否解析
			AppConfig appConfig = AppUtils.getAppConfigFromRoutingContext(ctx);
			ApiConfig apiConfig = AppUtils.getApiConfigFromRoutingContext(ctx);
			String apiServiceType = ctx.get(API_SERVICE_TYPE);
			String contentType = ctx.request().headers().get(HttpHeaders.CONTENT_TYPE);
			if (AppUtils.isAnalysisBody(appConfig, apiConfig, apiServiceType, contentType)) {
				String keyCircuitBreaker = appConfig.getAppCode() + CACHE_KEY_CONNECTOR + apiConfig.getApiCode();
				SacCircuitBreaker sacCircuitBreaker = AppConfigHandler.getApiCodeCircuitBreaker(keyCircuitBreaker);
				String body = AppUtils.isEnableDataSecurity(appConfig)
						? AppConfigHandler.bodyDecrypt(ctx.body().asString(), appConfig.getAppCode())
						: ctx.body().asString();
				HttpRequest<Buffer> requestBuffer = buildCallRequestBuffer(ctx, body, apiConfig);
				try {
					if (sacCircuitBreaker != null && sacCircuitBreaker.getCircuitBreaker() != null) {
						breakerCall(ctx, sacCircuitBreaker, requestBuffer, body);
					} else {
						normalCall(ctx, requestBuffer, body);
					}
				} catch (Exception e) {
					e.printStackTrace();
					int errorCode = 10014;
					if (e instanceof HttpException) {
						errorCode = ((HttpException) e).getStatusCode();
					}
					throw new HttpException(errorCode);
				}
			} else {
				ctx.next();
			}
		} catch (Exception e) {
			log.error("RouteRequestHandlerImpl:", e);
			ctx.fail(new ServiceException(GatewayError.DEFAULT_SERVICE_ERROR));
			return;
		}
	}
}
